#cs ----------------------------------------------------------------------------

 Copyright (C) 2014  Sebastian Walter (chesstiger)

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 AutoIt Version: 3.3.10.2

#ce ----------------------------------------------------------------------------

#include <WinAPI.au3>
#include <WindowsConstants.au3>

Global Const $__sITASKBARLIST_CLSID = "{56FDF344-FD6D-11D0-958A-006097C9A090}"
Global Const $__sITASKBARLIST_IID = "{ea1afb91-9e28-4b86-90e9-9e9f8a5eefaf}"
Global Const $__mtagTHUMBBUTTON = "long dwMask#; UINT iId#; UINT iBitmap#; handle hIcon#; wchar szTip#[260]; long dwFlags#;"
Global Const $__sITASKBARLIST_INTERFACE = _
        "QueryInterface long(ptr;ptr;ptr);" & _
        "AddRef ulong();" & _
        "Release ulong();" & _
        "HrInit long:cdecl();" & _
        "AddTab long(hwnd);" & _
        "DeleteTab long:cdecl(hwnd);" & _
        "ActivateTab long(hwnd);" & _
        "SetActiveAlt long(hwnd);" & _
        "MarkFullscreenWindow long(hwnd;int);" & _
        "SetProgressValue long:cdecl(hwnd;uint64;uint64);" & _
        "SetProgressState long(hwnd;int);" & _
        "RegisterTab long(hwnd;hwnd);" & _
        "UnregisterTab long(hwnd);" & _
        "SetTabOrder long(hwnd;hwnd);" & _
        "SetTabActive long(hwnd;hwnd;dword);" & _
        "ThumbBarAddButtons long(hwnd;uint;ptr);" & _
        "ThumbBarUpdateButtons long(hwnd;uint;ptr);" & _
        "ThumbBarSetImageList long(hwnd;ptr);" & _
        "SetOverlayIcon long(hwnd;ptr;wstr);" & _
        "SetThumbnailTooltip long(hwnd;wstr);" & _
        "SetThumbnailClip long(hwnd;ptr);"

Global Const $TBPF_NOPROGRESS = 0
Global Const $TBPF_INDETERMINATE = 0x1
Global Const $TBPF_NORMAL = 0x2
Global Const $TBPF_ERROR = 0x4
Global Const $TBPF_PAUSED = 0x8

Global Const $THB_BITMAP   = 0x00000001
Global Const $THB_ICON     = 0x00000002
Global Const $THB_TOOLTIP  = 0x00000004
Global Const $THB_FLAGS    = 0x00000008

Global Const $THBF_ENABLED         = 0x00000000
Global Const $THBF_DISABLED        = 0x00000001
Global Const $THBF_DISMISSONCLICK  = 0x00000002
Global Const $THBF_NOBACKGROUND    = 0x00000004
Global Const $THBF_HIDDEN          = 0x00000008
Global Const $THBF_NONINTERACTIVE  = 0x00000010

Global Const $THBN_CLICKED = 6144

Global Enum $__OLDWNDPROC, $__DUMMYS, $__BUTTONSTRUCT, $__JUSTUPDATE

Global $__oTBInterface = 0
Global $__oTBInfo = 0
Global $__hWndProc = 0
Global $__pWndProc = 0

; #FUNCTION# ====================================================================================================================
; Name ..........: _TBB_Init
; Description ...: Initializes the UDF objects and callbacks.
; Syntax ........: _TBB_Init()
; Parameters ....: None
; Return values .: None
; Author ........: chesstiger
; Modified ......:
; Remarks .......:
; Related .......: _TBB_Shutdown
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _TBB_Init()
	$__oTBInterface = ObjCreateInterface($__sITASKBARLIST_CLSID, $__sITASKBARLIST_IID, $__sITASKBARLIST_INTERFACE, False)
	$__oTBInterface.HrInit()
	$__oTBInfo = ObjCreate("Scripting.Dictionary")
	$__hWndProc = DllCallbackRegister("__WndProc", "ptr", "hwnd;uint;long;long")
	$__pWndProc = DllCallbackGetPtr($__hWndProc)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _TBB_Shutdown
; Description ...: Disposes the UDF objects and callbacks.
; Syntax ........: _TBB_Shutdown()
; Parameters ....: None
; Return values .: None
; Author ........: chesstiger
; Modified ......:
; Remarks .......:
; Related .......: _TBB_Init
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _TBB_Shutdown()
	For $hWnd In $__oTBInfo.Keys
		$aTBInfo = $__oTBInfo(String($hWnd))
		_WinAPI_SetWindowLong($hWnd, $GWL_WNDPROC, $aTBInfo[$__OLDWNDPROC])
	Next
	$__oTBInfo = 0
	$__oTBInterface = 0
	DllCallbackFree($__hWndProc)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _TBB_Create
; Description ...: Creates the basic array of THUMBBUTTON structures.
; Syntax ........: _TBB_Create($hWnd, $iButtonCount)
; Parameters ....: $hWnd                - The window handle to set the buttons for.
;                  $iButtonCount        - The count of buttons.
; Return values .: The THUMBBUTTON struct array (not required).
; Author ........: chesstiger
; Modified ......:
; Remarks .......:
; Related .......: _TBB_SetButton, _TBB_Update
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _TBB_Create($hWnd, $iButtonCount)
	Local $tagButtons = "", $tButtons, $aTBInfo[4], $aTBDummys[$iButtonCount]

	For $i = 1 To $iButtonCount
		$tagButtons &= StringReplace($__mtagTHUMBBUTTON, "#", $i)
	Next
	$tButtons = DllStructCreate($tagButtons)

	$aTBInfo[$__OLDWNDPROC] = _WinAPI_SetWindowLong($hWnd, $GWL_WNDPROC, $__pWndProc)
	$aTBInfo[$__DUMMYS] = $aTBDummys
	$aTBInfo[$__BUTTONSTRUCT] = $tButtons
	$aTBInfo[$__JUSTUPDATE] = False
	$__oTBInfo(String($hWnd)) = $aTBInfo

	Return $tButtons
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _TBB_SetButton
; Description ...: Sets the data for the specified button number.
; Syntax ........: _TBB_SetButton($hWnd, $iButton, $sTooltip, $iFlags[, $hIcon = 0])
; Parameters ....: $hWnd                - The window handle (same as by _TBB_CreateButton).
;                  $iButton             - The button number to set the data for.
;                  $sTooltip            - The mouseover text.
;                  $iFlags              - Flags
;                  |$THBF_ENABLED 			- The button is enabled.
;                  |$THBF_DISABLED 			- The button is disabled.
;                  |$THBF_DISMISSIONCLICK 	- The thumbbar will not close after a click.
;                  |$THBF_NOBACKGROUND	 	- The button has no background or border.
;                  |$THBF_HIDDEN		 	- The button is not visible.
;                  |$THBF_NOINTERACTIVE	 	- The button has no optical feedback on a click.
;                  $hIcon               - [optional] An icon handle to display. If 0, the corrisponding image from the ImageList is taken.
; Return values .: A control id, to handle the button events (can used for GUISetOnEvent or in GUIGetMsg loop).
; Author ........: chesstiger
; Modified ......:
; Remarks .......:
; Related .......: _TBB_Create, _TBB_Update
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _TBB_SetButton($hWnd, $iButton, $sTooltip, $iFlags, $hIcon = 0)
	$aTBInfo = $__oTBInfo(String($hWnd))
	$aTBDummys = $aTBInfo[$__DUMMYS]
	$tButtons = $aTBInfo[$__BUTTONSTRUCT]

	$iMask = BitOR($THB_TOOLTIP, $THB_FLAGS)
	If $hIcon Then
		$iMask = BitOR($iMask, $THB_ICON)
	Else
		$iMask = BitOR($iMask, $THB_BITMAP)
	EndIf

	DllStructSetData($tButtons, "dwMask" & $iButton, $iMask)
	DllStructSetData($tButtons, "iId" & $iButton, $iButton - 1)
	DllStructSetData($tButtons, "hIcon" & $iButton, $hIcon)
	DllStructSetData($tButtons, "iBitmap" & $iButton, $iButton - 1)
	DllStructSetData($tButtons, "szTip" & $iButton, $sTooltip)
	DllStructSetData($tButtons, "dwFlags" & $iButton, $iFlags)

	If Not $aTBInfo[$__JUSTUPDATE] Then
		$aTBDummys[$iButton - 1] = GUICtrlCreateDummy()
	EndIf

	$aTBInfo[$__DUMMYS] = $aTBDummys
	$__oTBInfo(String($hWnd)) = $aTBInfo

	Return $aTBDummys[$iButton - 1]
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _TBB_Update
; Description ...: Finally sends the data to Windows.
; Syntax ........: _TBB_Update($hWnd, $hImageList)
; Parameters ....: $hWnd                - The window handle (same as by _TBB_CreateButton).
;                  $hImageList          - An image list.
; Return values .: None
; Author ........: chesstiger
; Modified ......:
; Remarks .......:
; Related .......: _TBB_Create, _TBB_SetButton
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _TBB_Update($hWnd, $hImageList)
	$aTBInfo = $__oTBInfo(String($hWnd))
	$aTBDummys = $aTBInfo[$__DUMMYS]
	$iButtons = UBound($aTBDummys)
	$tButtons = $aTBInfo[$__BUTTONSTRUCT]

	$__oTBInterface.ThumbBarSetImageList($hWnd, $hImageList)

	If $aTBInfo[$__JUSTUPDATE] Then
		$__oTBInterface.ThumbBarUpdateButtons($hWnd, $iButtons, DllStructGetPtr($tButtons))
	Else
		$__oTBInterface.ThumbBarAddButtons($hWnd, $iButtons, DllStructGetPtr($tButtons))
		$aTBInfo[$__JUSTUPDATE] = True
		$__oTBInfo(String($hWnd)) = $aTBInfo
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _TBB_SetProgressValue
; Description ...: Sets the value of a taskbar progress bar.
; Syntax ........: _TBB_SetProgressValue($hWnd, $iPercent)
; Parameters ....: $hWnd                - Window handle
;                  $iPercent            - Percentage
; Return values .: None
; Author ........: chesstiger
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _TBB_SetProgressValue($hWnd, $iPercent)
	$__oTBInterface.SetProgressValue($hWnd, $iPercent, 100)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _TBB_SetProgressState
; Description ...: Sets the state of a taskbar progress bar.
; Syntax ........: _TBB_SetProgressState($hWnd, $iState)
; Parameters ....: $hWnd                - Window handle
;                  $iState              - $TBPF_* constant
; Return values .: None
; Author ........: chesstiger
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _TBB_SetProgressState($hWnd, $iState)
	$__oTBInterface.SetProgressState($hWnd, $iState)
EndFunc

Func __WndProc($hWnd, $iMsg, $iwParam, $ilParam)
	$aTBInfo = $__oTBInfo(String($hWnd))
	$pOldWndProc = $aTBInfo[$__OLDWNDPROC]
	$aTBDummys = $aTBInfo[$__DUMMYS]

	If $iMsg = $WM_COMMAND Then
		If _WinAPI_HiWord($iwParam) = $THBN_CLICKED Then
			$iIdClicked = _WinAPI_LoWord($iwParam)
			For $i = 0 To UBound($aTBDummys) - 1
				If $iIdClicked = $i Then
					GUICtrlSendToDummy($aTBDummys[$i])
					Return 0
				EndIf
			Next
		EndIf
	EndIf

	Return _WinAPI_CallWindowProc($pOldWndProc, $hWnd, $iMsg, $iwParam, $ilParam)
EndFunc